package de.stefandraegersoftware.clipboardobserver;

import java.awt.Toolkit;
import java.awt.datatransfer.Clipboard;
import java.awt.datatransfer.Transferable;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * Klasse ClipboardObserver
 * Dient zum berwachen der Zwischenablage und startet im Interval einen Event wenn bestimmter Inhalt vorhanden ist.
 *
 * @author Stefan Draeger Software
 *
 */
public class ClipboardObserver implements Runnable {

	private static boolean run = true;

	/**
	 * Instance der Klasse ClipboardObserver
	 */
	private static ClipboardObserver instance;

	/**
	 * {@link java.util.Map} mit den {@link IObservable} Objekten
	 */
	private Map<EClipboardDataFlavor, List<IObservable>> observableMap = new HashMap<EClipboardDataFlavor, List<IObservable>>();

	/**
	 * Privater Kontruktor
	 */
	private ClipboardObserver() {
		//diese Klasse als Thread starten.
		new Thread(this).start();
	}

	/**
	 * Liefert die zur laufzeit der Anwendung gltige Instanz der Klasse {@link ClipboardObserver}
	 * @return - eine Instanz der Klasse {@link ClipboardObserver}
	 */
	public static ClipboardObserver getInstance() {
		if (instance == null) {
			instance = new ClipboardObserver();
		}
		return instance;
	}

	/**
	 * Nimmt ein {@link IObservable} Objekt entgegen und erweitert die {@link java.util.List} um dieses Objekt.
	 * @param clipboardDataFlavour - {@link EClipboardDataFlavor} der Typ
	 * @param observable - {@link IObservable} das Objekt
	 */
	public void addObservable(EClipboardDataFlavor clipboardDataFlavour, IObservable observable) {
		List<IObservable> observableList = this.observableMap.get(clipboardDataFlavour);
		if (observableList == null) {
			observableList = new ArrayList<IObservable>();
		}
		observableList.add(observable);
		this.observableMap.put(clipboardDataFlavour, observableList);
	}

	@Override
	public void run() {
		while (run) {
			try {
				Clipboard clipboard = Toolkit.getDefaultToolkit().getSystemClipboard();
				Transferable data = clipboard.getContents(null);
				if (data != null) {
					for (EClipboardDataFlavor entry : observableMap.keySet()) {
						List<IObservable> observableList = observableMap.get(entry);
						if (data.isDataFlavorSupported(entry.getDataFlavor()) && !observableList.isEmpty()) {
							for (IObservable observable : observableList) {
								observable.doAction(entry,data);
							}
						}
					}
				}
			} catch (Exception e) {
				e.printStackTrace();
			}

			try {
				Thread.sleep(1000);
			} catch (InterruptedException e) {
				e.printStackTrace();
			}
		}
	}

	public static boolean isRun() {
		return run;
	}

	public static void setRun(boolean inRun) {
		run = inRun;
	}

}